//============================================================================
// ClipAid - Data recognition aid for the clipboard
//
// Copyright (C) 1997 Ziff-Davis Publishing Company
// First published in PC Magazine by Douglas Boling
//
// Revision History:
//
// 1.0   Initial Release
//
//============================================================================

//
#define dim(x) (sizeof(x) / sizeof(x[0]))   
//
// Profile String Names
//
#define     PRO_XPOS        "WinPosX"
#define     PRO_YPOS        "WinPosY"
#define     PRO_XSIZE       "WinSizeX"
#define     PRO_YSIZE       "WinSizeY"
#define     PRO_HCNT        "Count"
#define     PRO_ENABLE      "Enable"
#define     PRO_MODE        "ClipMode"
#define     PRO_SAFE        "ClipSafe"
#define     PRO_KNOWN       "KnownTypes"
#define     PRO_AUTORUN     "AutoRun"
#define     PRO_SCACTIVATE  "SCActivate"

#define    APPNAME          "ClipAid"
#define    TBICON           1
#define    ICM_OPEN         100

#define    MYMSG_TBNOTIFY   WM_USER + 1
#define    MYMSG_REFRESH    WM_USER + 2

#define    MAXHISTORY       64

#define    IDM_SHOW         12
#define    IDM_MENUCOPY     20

//----------------------------------------------------------------------------
// Include files
//----------------------------------------------------------------------------
#include <afxwin.h>
#include <afxdisp.h>
#include <afxole.h>

#include <shlobj.h>

#include "resource.h"
#include "ClipAid.h"
#include "VerAbout.h"

// Create an instance my application class
CMyApp myApp;

//****************************************************************************
//CMyApp member functions
//
//****************************************************************************

//----------------------------------------------------------------------------
// InitInstance is called once when the program is started.
//----------------------------------------------------------------------------
BOOL CMyApp::InitInstance ()
{
	// See if we already are running.
	if (FindWindow (NULL, APPNAME))
		return FALSE;

	// Init OLE libs so we can use OLE related MFC classes
	if (!AfxOleInit())
		return FALSE;

	// Set the key under which the state data will be saved in the registry.
	SetRegistryKey ("PC Magazine");

	// Create my main window dialog class with new op to make it static.
	m_pMainWnd = new CClipAidSheet;

	// Set main Window icon.
	m_pMainWnd->SetIcon (myApp.LoadIcon (APPNAME), TRUE);

	// Now set the default appearence for the dialog and force paint msg
	INT x = GetProfileInt ("Window", PRO_XPOS, GetSystemMetrics (SM_CXSCREEN)/2);
	INT y = GetProfileInt ("Window", PRO_YPOS, 100);
	x = min (x, GetSystemMetrics (SM_CXSCREEN));
	y = min (y, GetSystemMetrics (SM_CYSCREEN));
	m_pMainWnd->SetWindowPos (NULL, x, y, 0, 0, SWP_NOSIZE | 
		                      SWP_HIDEWINDOW | SWP_NOZORDER);  //Hide, since we are a task bar icon!

	m_pMainWnd->UpdateWindow ();
	return TRUE;
}

//----------------------------------------------------------------------------
// ExitInstance is called once when the program is terminated.
//----------------------------------------------------------------------------
int CMyApp::ExitInstance ()
{

	return 0;
}
//----------------------------------------------------------------------------
// ClipAidSheet Idle Routine - Return FALSE if no more idle calls needed.
//----------------------------------------------------------------------------
BOOL CMyApp::OnIdle (LONG lCount)
{
	BOOL bContinue;
	CClipAidSheet *pDlgWnd;

	// Call old Idle call for MFC
	bContinue = CWinApp::OnIdle (lCount);

	// Save program state data
	pDlgWnd = (CClipAidSheet *)m_pMainWnd;
	if (pDlgWnd->m_fDirty) {
		if (lCount > 500) {
			pDlgWnd->SaveState ();
			pDlgWnd->m_fDirty = FALSE;
		} else
			bContinue = TRUE;
	}
	return bContinue;
} 
//****************************************************************************
//CClipAidSheet member functions
//
//****************************************************************************

// MFC macro that creates the message dispatch table for the dialog
BEGIN_MESSAGE_MAP (CClipAidSheet, CPropertySheet)
	ON_WM_CREATE ()
	ON_WM_CLOSE ()
	ON_WM_CHANGECBCHAIN ()
	ON_WM_DRAWCLIPBOARD ()
	ON_WM_ENDSESSION ()

	ON_MESSAGE (MYMSG_TBNOTIFY, OnTaskBarNotify)
	
	ON_COMMAND (IDM_SHOW, OnMenuShow)
	ON_COMMAND (IDD_ENABLE, OnMenuEnable)
	ON_COMMAND_RANGE (IDM_MENUCOPY, IDM_MENUCOPY+4, OnMenuCopy)
	ON_BN_CLICKED (IDOK, OnCancel)
	ON_BN_CLICKED (IDCANCEL, OnCancel)
	ON_BN_CLICKED (ID_APPLY_NOW, OnCancel)
END_MESSAGE_MAP ()
				
//----------------------------------------------------------------------------
// ClipAidSheet Constructor
//----------------------------------------------------------------------------
CClipAidSheet::CClipAidSheet (CWnd* pParentWnd) : CPropertySheet ("ClipAid", pParentWnd)
{
	m_hwndNextClip = 0;

	m_HistoryPage.m_fEnableOnce = FALSE;
	m_ConfigPage.m_nEnable = myApp.GetProfileInt ("Config", PRO_MODE, 2);
	m_ConfigPage.m_fEnable = myApp.GetProfileInt ("Config", PRO_ENABLE, TRUE);
	m_ConfigPage.m_fKnownOnly = myApp.GetProfileInt ("Config", PRO_KNOWN, FALSE);
	m_ConfigPage.m_fSafeClip = myApp.GetProfileInt ("Config", PRO_SAFE, TRUE);

	m_ConfigPage.m_fAutoRun = myApp.GetProfileInt ("Config", PRO_AUTORUN, FALSE);
	m_ConfigPage.m_fSCActivate = myApp.GetProfileInt ("Config", PRO_SCACTIVATE, FALSE);

	m_ConfigPage.m_hIconEnable = myApp.LoadIcon (APPNAME);
	m_ConfigPage.m_hIconDisable = myApp.LoadIcon (IDI_DISABLE);

	// Register clipboard formats of intrest.
	m_cfFilename = ::RegisterClipboardFormat ("FileName");
	m_cfURL = ::RegisterClipboardFormat ("UniformResourceLocator");
	m_cfFileContents = ::RegisterClipboardFormat ("FileContents");
	m_cfFileGroupDescriptor = ::RegisterClipboardFormat ("FileGroupDescriptor");
	m_cfOLEDataObject = ::RegisterClipboardFormat ("DataObject");
	m_cfRichText = ::RegisterClipboardFormat ("Rich Text Format");

	m_fDirty = FALSE;
	// init history buffer
	{
		INT i;
		char szKey[16];

		m_HistoryPage.m_sHistoryCnt = myApp.GetProfileInt ("History", PRO_HCNT, 0);
		m_HistoryPage.m_saHistory.SetSize (m_HistoryPage.m_sHistoryCnt);
		for (i = 0; i < m_HistoryPage.m_sHistoryCnt; i++) {
			::wsprintf (szKey, "%d", i);
			m_HistoryPage.m_saHistory[i] = myApp.GetProfileString ("History", szKey);
		}
	}
	AddPage (&m_HistoryPage);
	AddPage (&m_ConfigPage);
	AddPage (&m_AboutPage);

	// Actually create the dialog box.
	// Override the default styles to remove the WS_VISIBLE flag
	Create (NULL, DS_MODALFRAME | DS_3DLOOK | DS_CONTEXTHELP |
		    DS_SETFONT | WS_POPUP | WS_CAPTION | WS_SYSMENU);
}
//----------------------------------------------------------------------------
// ClipAidSheet Destructor
//----------------------------------------------------------------------------
CClipAidSheet::~CClipAidSheet (void)
{
	return;
}
//----------------------------------------------------------------------------
// OnInitDialog - Called during dialog box init
//----------------------------------------------------------------------------
BOOL CClipAidSheet::OnInitDialog ()
{
	BOOL rc;

	//
	// First, call base class function.  Before calling, set modless
	// flag to false, to fool it into leaving the window size large 
	// enough for the buttons below the property pages.  These are
	// normally, removed in modeless dialogs.
	//
	m_bModeless = FALSE;
	rc = CPropertySheet::OnInitDialog ();
	m_bModeless = TRUE;

	m_fFlags = 0;

	m_ConfigPage.m_hSheet = m_hWnd;

	HWND hWnd = ::GetDlgItem(m_hWnd, IDOK);
	::ShowWindow (hWnd, SW_HIDE);
	::EnableWindow (hWnd, FALSE);

	// Hide the cancel button
	hWnd = ::GetDlgItem(m_hWnd, IDCANCEL);
	ASSERT(hWnd != NULL);
	::ShowWindow (hWnd, SW_HIDE);

	// Show the 'Apply' button, but change its text to Close.
	hWnd = ::GetDlgItem(m_hWnd, ID_APPLY_NOW);
	ASSERT(hWnd != NULL);
	::SetWindowText (hWnd, "Close");
	::ShowWindow (hWnd, SW_SHOW);
	::EnableWindow (hWnd, TRUE);
	// Hook from the clipboard viewer chain.
	m_hwndNextClip = SetClipboardViewer ();

	// Insert taskbar icon.
	NOTIFYICONDATA nid;

	nid.cbSize = sizeof (nid);
	nid.hWnd = m_hWnd;
	nid.uID = TBICON;
	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	nid.uCallbackMessage = MYMSG_TBNOTIFY;
	if (m_ConfigPage.m_fEnable) {
		nid.hIcon = m_ConfigPage.m_hIconEnable;
		lstrcpy (nid.szTip, "ClipAid is active");
	} else {
		nid.hIcon = m_ConfigPage.m_hIconDisable;
		lstrcpy (nid.szTip, "ClipAid is disabled");
	}
	::Shell_NotifyIcon (NIM_ADD, &nid);
	return rc;
}
//----------------------------------------------------------------------------
// OnEndSession - End Session message handler
//----------------------------------------------------------------------------
void CClipAidSheet::OnEndSession (BOOL bEnding)
{
	if (bEnding && m_fDirty)
		SaveState ();
	return;
}
//----------------------------------------------------------------------------
// OnMenuShow - Show window from context menu cmd
//----------------------------------------------------------------------------
void CClipAidSheet::OnMenuShow ()
{
	ShowWindow (SW_SHOW);
	return;
}
//----------------------------------------------------------------------------
// OnMenuEnable - Enable clipboard monitoring from context menu cmd
//----------------------------------------------------------------------------
void CClipAidSheet::OnMenuEnable ()
{
	m_ConfigPage.Enable(m_hWnd);
	return;
}
//----------------------------------------------------------------------------
// OnMenuCopy - Copy history item to the clipboard
//----------------------------------------------------------------------------
void CClipAidSheet::OnMenuCopy (UINT nID)
{
	m_HistoryPage.CopyItem(nID - IDM_MENUCOPY);
	return;
}
//----------------------------------------------------------------------------
// OnOK -  Called when PS is dismissed via sys menu or Exit.  
//         Must be overridden for Modeless dialogs
//----------------------------------------------------------------------------
void CClipAidSheet::OnClose ()
{
	ShowWindow (SW_HIDE);
}
//----------------------------------------------------------------------------
// OnCancel - Called when esc key is prssed.  
//            Must be overridden for Modeless dialogs
//----------------------------------------------------------------------------
void CClipAidSheet::OnCancel ()
{
	ShowWindow (SW_HIDE);
}
//----------------------------------------------------------------------------
// OnDestroy - Called when the window is destroyed
//----------------------------------------------------------------------------
BOOL CClipAidSheet::DestroyWindow () 
{
	WINDOWPLACEMENT wp;

	// Save window position
	wp.length = sizeof (WINDOWPLACEMENT);
	GetWindowPlacement (&wp);
	myApp.WriteProfileInt ("Window", PRO_XPOS, wp.rcNormalPosition.left);
	myApp.WriteProfileInt ("Window", PRO_YPOS, wp.rcNormalPosition.top);

	// Unhook from the clipboard viewer chain.
	ChangeClipboardChain (m_hwndNextClip);

	return CPropertySheet::DestroyWindow ();
}
//----------------------------------------------------------------------------
// PostNCDestroy - Called after window destroyed
//            Must be overridden for Modeless dialogs
//----------------------------------------------------------------------------
void CClipAidSheet::PostNcDestroy () 
{
	// Since the dialog box is modeless and the dialog object was created 
	// with a new operator, we need to delete ourselves.
	delete this;
}
//----------------------------------------------------------------------------
// OnTaskBarNotify - Called on event from taskbar icon
//----------------------------------------------------------------------------
LONG CClipAidSheet::OnTaskBarNotify (UINT wParam, LONG lParam) 
{
	UINT uID = wParam;
	UINT uMessage = lParam;

	switch (uMessage) {

	// On right button, display context menu
	case WM_RBUTTONDOWN:
		{
		CMenu pop, sub;
		POINT pt;

		GetCursorPos (&pt);
		// Create context menu from scratch
		pop.CreateMenu ();
		sub.CreatePopupMenu ();
		sub.AppendMenu (MF_STRING, IDM_SHOW, "P&roperties");
		{
			int i, sCnt;
			sCnt = min (m_HistoryPage.m_sHistoryCnt, 4);
			if (sCnt) {
				sub.AppendMenu (MF_SEPARATOR);
				for (i = 0; i < sCnt; i++)
					sub.AppendMenu (MF_STRING, IDM_MENUCOPY+i, (LPCSTR)m_HistoryPage.m_saHistory[i]);
			}

		}
		pop.AppendMenu (MF_POPUP, (UINT) sub.Detach());
		CMenu *pContext = pop.GetSubMenu (0);

		// This is the first part of a hack necessary by Win95 to 
		// display context menus from the task bar
		::SetForegroundWindow (m_hWnd);
		// Display menu between hacks...
		pContext->TrackPopupMenu (TPM_LEFTALIGN | TPM_LEFTBUTTON | 
			                      TPM_RIGHTBUTTON, pt.x, 
		                          pt.y, this);
		// This is the second part of a hack necessary by Win95 to 
		// display context menus from the task bar.  This is just
		// a dummy message to the window.
		PostMessage (WM_USER + 10, 0, 0);
		}
		break;

	// On left double click, show main window
	case WM_LBUTTONDBLCLK:
		if (!m_ConfigPage.m_fSCActivate) {
			ShowWindow (SW_SHOW);
			::SetForegroundWindow (m_hWnd);
		}
		break;

	// On left click and special flag set, show main window
	case WM_LBUTTONUP:
		if (m_ConfigPage.m_fSCActivate) {
			ShowWindow (SW_SHOW);
			::SetForegroundWindow (m_hWnd);
		}
		break;

	}

	return 0;
}
//----------------------------------------------------------------------------
// OnChangeCbChain - Called when clipboard chain has changed
//----------------------------------------------------------------------------
void CClipAidSheet::OnChangeCbChain (HWND hWndRemove, HWND hWndAfter) 
{
	//Modify local storage
	if (m_hwndNextClip == hWndRemove)
		m_hwndNextClip = hWndAfter;
	//Pass message along
	::SendMessage (m_hwndNextClip, WM_CHANGECBCHAIN, (WPARAM)hWndRemove, 
	               (LPARAM)hWndAfter);
	return;
}
//----------------------------------------------------------------------------
// OnDrawClipboard - Called in response to a WM_DRAWCLIPBOARD message  
//----------------------------------------------------------------------------
void CClipAidSheet::OnDrawClipboard (void)
{
	HGLOBAL hClip;
	LPSTR lpClipText;
	LONG lDataSize;													
	INT sClipType = 0;
	char szText[256], szURL[280];

	// Don't re-enter.
	if (m_fFlags & FLAG_ACTIVE) {
		::SendMessage (m_hwndNextClip, WM_DRAWCLIPBOARD, 0, 0);
		return;
	}
		
	m_fFlags |= FLAG_ACTIVE;
	if (OpenClipboard()) { 
		// See if no URL in clipboard
		if (!::IsClipboardFormatAvailable(m_cfURL)) {

			hClip = GetClipboardData(CF_TEXT); 
			// If text in clipboard			
			if (hClip) {
				lDataSize = GlobalSize (hClip);
				lDataSize = GlobalSize (hClip);
				if (lDataSize < 256) {
					lpClipText = (LPSTR)GlobalLock(hClip); 

					sClipType = WhatURL (lpClipText, szURL);
					lstrcpy (szText, lpClipText);
				}
				GlobalUnlock (hClip);
			} 
		}
		CloseClipboard(); 
		
		if (m_ConfigPage.m_fEnable || m_HistoryPage.m_fEnableOnce) { 
			m_HistoryPage.m_fEnableOnce = FALSE;
			if ((sClipType) &&
			    // Bail if complex data in clipboard
			    !(m_ConfigPage.m_fSafeClip && ::IsClipboardFormatAvailable(m_cfOLEDataObject)))
				AddURLToClip (szText, szURL, sClipType);
		}
	}
	m_fFlags &= ~FLAG_ACTIVE;

	if (sClipType || ::IsClipboardFormatAvailable(m_cfURL)) {
		// If we see a url in the clipboard, get it for the 
		// history list.
		if (!sClipType) {
			if (OpenClipboard()) { 
				hClip = GetClipboardData(m_cfURL); 
				lpClipText = (LPSTR)GlobalLock (hClip);
				lstrcpyn (szURL, lpClipText, sizeof (szURL));
				GlobalUnlock (hClip);
				CloseClipboard(); 
			}
		}	
				
		// Add url to history list
		{
			INT i, sCount;

			sCount = m_HistoryPage.m_sHistoryCnt;
			// Remove any duplicates
			for (i = 0; i < sCount; i++) {
				if (lstrcmp (szURL, (LPCSTR)m_HistoryPage.m_saHistory[i]) == 0) {
					m_HistoryPage.m_saHistory.RemoveAt(i);
					sCount--;
					break;
				}
			}
			// Cap size of array
			if (sCount >= MAXHISTORY-1) {
				m_HistoryPage.m_saHistory.RemoveAt(sCount);
				sCount--;
			}
			// Insert new value at base
			m_HistoryPage.m_saHistory.InsertAt(0, szURL);
			m_HistoryPage.m_sHistoryCnt = sCount + 1;
			m_HistoryPage.PostMessage (MYMSG_REFRESH, 0, 0);

			// Set dirty flag so idle loop will auto save history list.
			m_fDirty = TRUE;
		}
	}

	// Pass along drawclipboard message to next in chain.
	if (m_hwndNextClip)
		::SendMessage (m_hwndNextClip, WM_DRAWCLIPBOARD, 0, 0);
	return;
}
//----------------------------------------------------------------------------
// SaveState - Saves the state data
//----------------------------------------------------------------------------
INT CClipAidSheet::SaveState (void)
{
	INT i;
	char szKey[16];

	myApp.WriteProfileString ("History", NULL, NULL);
	for (i = 0; i < m_HistoryPage.m_sHistoryCnt; i++) {
		::wsprintf (szKey, "%d", i);
		myApp.WriteProfileString ("History", szKey, m_HistoryPage.m_saHistory[i]);
	}
	myApp.WriteProfileInt ("History", PRO_HCNT, m_HistoryPage.m_sHistoryCnt);

	myApp.WriteProfileInt ("Config", PRO_MODE, m_ConfigPage.m_nEnable);
	myApp.WriteProfileInt ("Config", PRO_ENABLE, m_ConfigPage.m_fEnable);
	myApp.WriteProfileInt ("Config", PRO_KNOWN, m_ConfigPage.m_fKnownOnly);
	myApp.WriteProfileInt ("Config", PRO_SAFE, m_ConfigPage.m_fSafeClip);
	myApp.WriteProfileInt ("Config", PRO_AUTORUN, m_ConfigPage.m_fAutoRun);
	myApp.WriteProfileInt ("Config", PRO_SCACTIVATE, m_ConfigPage.m_fSCActivate);

	return 0;
}
//----------------------------------------------------------------------------
// WhatURL - Determines the type of url captured.
//----------------------------------------------------------------------------
#define URL_WEB         1
#define URL_FTP         2
#define URL_FILE        3
#define URL_GOPHER      4
#define URL_MAIL        5
#define URL_NEWS        6
#define URL_TELNET      7

INT CClipAidSheet::WhatURL (LPSTR lpszStr, LPSTR szOut) 
{
	INT i, rc = 0;
	LPSTR lpStart, lpTemp;
	BOOL fScheme = FALSE;
	LPCSTR lpSch[] = {"http", "ftp", "file", "gopher", "mailto", 
	                  "news", "telnet"};

	// Assume not URL if over 256 chars in length.
	if (lstrlen (lpszStr) > 256)
		return 0;

	//Lower case string for future comparisons
	strlwr (lpszStr);

	//Trim off leading and trailing spaces
	for (lpStart = lpszStr; *lpStart && (*lpStart <= ' '); lpStart++);
	if (*lpStart) {
		for (lpTemp = lpStart + 1; *lpTemp && (*lpTemp > ' '); lpTemp++);
		*lpTemp = '\0';
	}
	// See if url has scheme prefix
	lpTemp = strstr (lpStart, "://");
	if (lpTemp) {
		char ch;

		ch = *lpTemp;
		*lpTemp = '\0';

		for (i = 0; i < dim (lpSch); i++)
			if (lstrcmp (lpStart, lpSch[i]) == 0)
				break;
	
		if (i < dim (lpSch)) {
			rc = i + 1;
			fScheme = TRUE;
		} 
		*lpTemp = ch;
	}
	// if no scheme, see if www. prefix
	if (rc == 0) {
		lpTemp = strchr (lpStart, '.');
		if (lpTemp) {
			char ch;

			ch = *lpTemp;
			*lpTemp = '\0';

			if (lstrcmp (lpStart, "www") == 0)
				rc = URL_WEB;

			*lpTemp = ch;
		}
	}
	// if no www, see if mail address
	if (rc == 0) {
		lpTemp = strchr (lpStart, '@');
		// At sign and not in first or last char
		if ((lpTemp) && (lpStart - lpTemp) && 
			(lpStart + lstrlen (lpStart) - 1 != lpTemp))
			rc = URL_MAIL;

	}
	if (rc) {
		if (!fScheme) {
			lstrcpy (szOut, lpSch[rc-1]);
			lstrcat (szOut, "://");
		} else
			szOut[0] = '\0';
		lstrcat (szOut, lpStart);
	}
	return rc;
}

//----------------------------------------------------------------------------
// CacheClipData - Saves the current clipboard contents.
//----------------------------------------------------------------------------
INT CClipAidSheet::CacheClipData (HGLOBAL *hMem, BOOL fKnownOnly)
{
	INT i, sCnt;
	UINT uFormat;
	LPMYCLIPENTRY lpEntry;
	DWORD dwClipTypes = 0, dwClipFlag;
	HGLOBAL hClip;
	LPCLIPCACHE lpCC;

	OpenClipboard();

	sCnt = ::CountClipboardFormats ();

	//Alloc mem block to hold array;
	*hMem = GlobalAlloc (GHND, (sCnt * sizeof (MYCLIPENTRY)) + sizeof (INT));
	if (!*hMem)
		return 0;
	lpCC = (LPCLIPCACHE) GlobalLock (*hMem);
	if (!lpCC)
		return 0;

	//Init array of clipboard data values
	lpCC->sCnt = 0;
	lpEntry = &lpCC->ce;

	// Get each clipboard data type
	uFormat = 0;
	for (i = 0; i < sCnt; i++) {
		 uFormat = ::EnumClipboardFormats (uFormat);
		if (uFormat == 0)
			break;

		// Don't save the clip types we are replacing.
		if ((uFormat == m_cfOLEDataObject) || (uFormat == m_cfFileGroupDescriptor) ||
		    (uFormat == m_cfFileContents) || (uFormat == m_cfURL))
			continue;						  

		// Convert format to a bit flag.
		dwClipFlag = 0;
		if (uFormat < 32) {
			dwClipFlag = (DWORD)1 << uFormat;
		// If this is a non-predefined cf format and we're only saving known
		// types, then check for known types.
		} else if (fKnownOnly && (uFormat != m_cfRichText))
			continue;

		// See if we already have this type in the array.  This
		// prevents auto generated types from being stored in array.
		if (!(dwClipTypes & dwClipFlag)) {
			dwClipTypes |= dwClipFlag;

			// Mask off auto generated types created by Windows.  The original
			// data type is always presented first, the others are created and
			// therefore don't need to be stored.
			switch (uFormat) {
				case CF_TEXT:
					dwClipTypes |= ((DWORD)1 << CF_OEMTEXT) | ((DWORD)1 << CF_UNICODETEXT);
					dwClipTypes |= (DWORD)1 << CF_LOCALE;
					break;
				case CF_BITMAP:
					dwClipTypes |= (DWORD)1 << CF_DIB;
					break;
				case CF_METAFILEPICT:
					dwClipTypes |= (DWORD)1 << CF_ENHMETAFILE;
					break;
				case CF_OEMTEXT:    //     7
					dwClipTypes |= ((DWORD)1 << CF_TEXT) | ((DWORD)1 << CF_UNICODETEXT);
					dwClipTypes |= (DWORD)1 << CF_LOCALE;
					break;
				case CF_DIB:        //     8
					dwClipTypes |= (DWORD)1 << CF_BITMAP;
					break;
				case CF_UNICODETEXT: //     13
					dwClipTypes |= ((DWORD)1 << CF_TEXT) | ((DWORD)1 << CF_OEMTEXT);
					dwClipTypes |= (DWORD)1 << CF_LOCALE;
					break;
				case CF_ENHMETAFILE: //     14
					dwClipTypes |= (DWORD)1 << CF_METAFILEPICT;
					break;
			}

			lpEntry->uFormat = uFormat;
			// Get the clipboard data, then copy since once we
			// clear the clipboard, this handle is invalid.
			hClip = GetClipboardData (lpEntry->uFormat);
			if (hClip) {
				INT nSize;
				LPBYTE lpClip, lpDest;

				nSize = GlobalSize (hClip);
				lpEntry->hData = GlobalAlloc (GHND | GMEM_DDESHARE, nSize);

				lpClip = (LPBYTE)GlobalLock (hClip);
				lpDest = (LPBYTE)GlobalLock (lpEntry->hData);
				memcpy (lpDest, lpClip, nSize);
				GlobalUnlock (lpEntry->hData);
				GlobalUnlock (hClip);

				lpEntry++;
				lpCC->sCnt++;
			}
		}
	}
	CloseClipboard();
	GlobalUnlock (*hMem);

	return lpCC->sCnt;
}
//----------------------------------------------------------------------------
// AddURLToClip - Insert data into the clipboard via a COleDataSource object
//----------------------------------------------------------------------------
void CClipAidSheet::AddURLToClip (LPSTR lpText, LPSTR lpURL, INT sType)
{
	static char szTag[] = {"[InternetShortcut]\r\nURL="};
	HGLOBAL hClip;
	LPSTR lpClip, lpTemp;
	LPFILEGROUPDESCRIPTOR lpData;
	INT nFileSize, nClipTypeCnt;
	COleDataSource *clipdata;
	HGLOBAL hOldClipArray;

	// Get current clipboard data
	nClipTypeCnt = CacheClipData (&hOldClipArray, m_ConfigPage.m_fKnownOnly);

	// Create data source object
	clipdata = new COleDataSource;

	// Add URL
	hClip = GlobalAlloc (GHND | GMEM_DDESHARE, 1024);
	lpClip = (LPSTR)GlobalLock (hClip);

	lstrcpy (lpClip, lpURL);
	*(LPWORD)(lpClip + lstrlen (lpClip)) = 0;

	GlobalUnlock (hClip); 
	clipdata->CacheGlobalData (m_cfURL, hClip); 

	// Add FileGroup data
	hClip = GlobalAlloc (GHND | GMEM_DDESHARE, 1024);
	lpData = (LPFILEGROUPDESCRIPTOR) GlobalLock (hClip);

	lpData->cItems = 1;
	lpData->fgd[0].dwFlags = FD_LINKUI | FD_FILESIZE;
	lpData->fgd[0].nFileSizeHigh = 0;
	lpData->fgd[0].nFileSizeLow = lstrlen (lpURL) + lstrlen (szTag) + 2;
	// Remove the scheme prefix from the url.
	lpTemp = strstr (lpURL, "://");
	if (lpTemp)
		lpTemp += 3;
	else
		lpTemp = lpURL;
	lstrcpy (lpData->fgd[0].cFileName, lpTemp);

	// Now remove any offending chars from the filename
	for (lpTemp = lpData->fgd[0].cFileName; *lpTemp; lpTemp++)
		if ((*lpTemp <= ' ') || (strchr ("\\/:?*<>|\'\".", *lpTemp)))
			*lpTemp = '_';
	
	lstrcat (lpData->fgd[0].cFileName, ".url");
	*(LPWORD)(lpData->fgd[0].cFileName + lstrlen (lpData->fgd[0].cFileName)) = 0;

	GlobalUnlock (hClip); 
	clipdata->CacheGlobalData(m_cfFileGroupDescriptor, hClip); 

	// Add File Contents data
	hClip = GlobalAlloc (GHND | GMEM_DDESHARE, 1024);
	lpClip = (LPSTR)GlobalLock (hClip);

	lstrcpy (lpClip, szTag);

	// Fix problem in mailto format for .url files.  Don't use ://, instead use : only!
	if (sType == URL_MAIL) {
		lstrcat (lpClip, "mailto:");
		lpTemp = strstr (lpURL, "://");
		if (lpTemp)
			lpTemp += 3;
		else
			lpTemp = lpURL;
		lstrcat (lpClip, lpTemp);
	} else 
		lstrcat (lpClip, lpURL);

	*(LPWORD)(lpClip + lstrlen (lpClip)) = 0;
	nFileSize = lstrlen (lpClip);

	GlobalUnlock (hClip); 
	clipdata->CacheGlobalData(m_cfFileContents, hClip); 

	// Restore old clip types
	if (hOldClipArray) {
		LPCLIPCACHE lpCC = (LPCLIPCACHE) GlobalLock (hOldClipArray);
		if (lpCC) {
			// Get ptr to first in array
			LPMYCLIPENTRY lpEntry = &lpCC->ce;
			INT i;
			for (i = 0; i < nClipTypeCnt; i++) {
				clipdata->CacheGlobalData(lpEntry->uFormat, lpEntry->hData); 
				lpEntry++;
			}
			GlobalUnlock (hOldClipArray);
		}
		GlobalFree (hOldClipArray);
	}
	clipdata->SetClipboard(); 

	return;
}
//****************************************************************************
//CConfigPage member functions
//
//****************************************************************************

// MFC macro that creates the message dispatch table for the dialog
BEGIN_MESSAGE_MAP (CConfigPage, CPropertyPage)

	ON_CONTROL_RANGE (BN_CLICKED, IDD_CLIPSAFE1, IDD_CLIPSAFE4, OnBtnClipSafe)
	ON_BN_CLICKED (IDD_AUTORUN, OnBtnAutoRun)
	ON_BN_CLICKED (IDD_REMOVE1, OnBtnRemove)
	ON_BN_CLICKED (IDD_SCACTIVATE, OnBtnSCActivate)
	ON_BN_CLICKED (IDD_CONFIGHELP, OnBtnHelp)

END_MESSAGE_MAP ()	
//----------------------------------------------------------------------------
// OnInitDialog - Called during dialog box init
//----------------------------------------------------------------------------
BOOL CConfigPage ::OnInitDialog ()
{
	CheckRadioButton (IDD_CLIPSAFE1, IDD_CLIPSAFE4, IDD_CLIPSAFE1 + m_nEnable);
	BtnAutoRun().SetCheck (m_fAutoRun ? 1 : 0);
	BtnSCActivate().SetCheck (m_fSCActivate ? 1 : 0);
	return TRUE;
}
//----------------------------------------------------------------------------
// OnBtnClipSafe - Handles array of radio buttons
//----------------------------------------------------------------------------
void CConfigPage::OnBtnClipSafe (UINT uID)
{

	m_nEnable = uID - IDD_CLIPSAFE1;
	switch (uID) {
		// Full clipboard stuff
		case IDD_CLIPSAFE1:
			if (!m_fEnable)
				Enable (m_hSheet);
			m_fKnownOnly = FALSE;
			m_fSafeClip = FALSE;
			break;

		// Stuff only RTF and text
		case IDD_CLIPSAFE2:
			if (!m_fEnable)
				Enable (m_hSheet);
			m_fKnownOnly = TRUE;
			m_fSafeClip = FALSE;
			break;

		// Stuff only if no DataObject in clip
		case IDD_CLIPSAFE3:
			if (!m_fEnable)
				Enable (m_hSheet);
			m_fKnownOnly = FALSE;
			m_fSafeClip = TRUE;
			break;

		// Never stuff
		case IDD_CLIPSAFE4:
			if (m_fEnable)
				Enable (m_hSheet);
			m_fKnownOnly = FALSE;
			m_fSafeClip = FALSE;
			break;
	}
	return;
}
//----------------------------------------------------------------------------
// Enable - Called to enable or disable the clipboard monitoring
//----------------------------------------------------------------------------
void CConfigPage::Enable (HWND hWnd)
{
	NOTIFYICONDATA nid;

	// Modify taskbar icon.
	nid.cbSize = sizeof (nid);
	nid.hWnd = hWnd;
	nid.uID = TBICON;
	nid.uFlags = NIF_ICON | NIF_TIP;
	nid.uCallbackMessage = 0;

	if (m_fEnable) {
		m_fEnable = FALSE;
		if (::IsWindow(m_hWnd))
			CheckRadioButton (IDD_CLIPSAFE1, IDD_CLIPSAFE4, IDD_CLIPSAFE4);
		nid.hIcon = m_hIconDisable;
		lstrcpy (nid.szTip, "ClipAid is disabled");
	} else {
		m_fEnable = TRUE;
		if (::IsWindow(m_hWnd))
			CheckRadioButton (IDD_CLIPSAFE1, IDD_CLIPSAFE4, IDD_CLIPSAFE1 + m_nEnable);
		nid.hIcon = m_hIconEnable;
		lstrcpy (nid.szTip, "ClipAid is active");
	}
	::Shell_NotifyIcon (NIM_MODIFY, &nid);

}
//----------------------------------------------------------------------------
// OnBtnEnable - Enable/Disable button clicked
//----------------------------------------------------------------------------
void CConfigPage::OnBtnEnable ()
{
	Enable(m_hSheet);
	return;
}
//----------------------------------------------------------------------------
// OnBtnAutoRun - Enable auto run of prog on startup
//----------------------------------------------------------------------------
void CConfigPage::OnBtnAutoRun ()
{
	char szName[MAX_PATH];
	HKEY hKey;
	LONG rc;

	GetModuleFileName (myApp.m_hInstance, szName, sizeof (szName));
	rc = RegOpenKey (HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Run", &hKey);
	if (rc == ERROR_SUCCESS) {
		if (m_fAutoRun)
			rc = RegDeleteValue (hKey, "ClipAid");
		else 
			rc = RegSetValueEx (hKey, "ClipAid", 0, REG_SZ, (CONST BYTE *)szName, lstrlen (szName));
		m_fAutoRun = ~m_fAutoRun;
		RegCloseKey (hKey);
	}
	return;
}
//----------------------------------------------------------------------------
// OnBtnRemove - Remove icon from taskbar and exit program
//----------------------------------------------------------------------------
void CConfigPage::OnBtnRemove ()
{
	NOTIFYICONDATA nid;

	// Remove icon from taskbar
	nid.cbSize = sizeof (nid);
	nid.hWnd = m_hSheet;
	nid.uID = TBICON;
	nid.uFlags = 0;
	nid.uCallbackMessage = 0;
	nid.hIcon = 0;
	nid.szTip[0] = '\0';
	BOOL fDone = ::Shell_NotifyIcon (NIM_DELETE, &nid);

	::DestroyWindow (m_hSheet);
}
//----------------------------------------------------------------------------
// OnBtnKnownOnly - Single Click checkbox handler
//----------------------------------------------------------------------------
void CConfigPage::OnBtnSCActivate ()
{
	m_fSCActivate = ~m_fSCActivate;
	return;
}
//----------------------------------------------------------------------------
// OnBtnHelp - Help button handler
//----------------------------------------------------------------------------
void CConfigPage::OnBtnHelp ()
{
	CTipsDialog dlg (this);

	dlg.DoModal();
	return;
}

//****************************************************************************
//CHistoryPage member functions
//
//****************************************************************************

// MFC macro that creates the message dispatch table for the dialog
BEGIN_MESSAGE_MAP (CHistoryPage, CPropertyPage)								

	ON_MESSAGE (MYMSG_REFRESH, OnRefresh)

	ON_LBN_DBLCLK (IDD_HISTORY, OnListHistory)
	ON_BN_CLICKED (IDD_COPY, OnBtnCopy)
	ON_BN_CLICKED (IDD_COPYALL, OnBtnCopyAll)
	ON_BN_CLICKED (IDD_DELETE, OnBtnDelete)
	ON_BN_CLICKED (IDD_RESET, OnBtnReset)

END_MESSAGE_MAP ()	
				

//----------------------------------------------------------------------------
// OnInitDialog - Called during dialog box init
//----------------------------------------------------------------------------
BOOL CHistoryPage::OnInitDialog ()
{
	PostMessage (MYMSG_REFRESH, 0, 0);
	return TRUE;
}
//----------------------------------------------------------------------------
// CopyItem - copies a url into the clipboard
//----------------------------------------------------------------------------
void CHistoryPage::CopyItem (INT nItem) 
{
	INT sLen;
	HGLOBAL hClip;
	LPSTR lpClip;

	if (nItem > m_sHistoryCnt - 1)
		return;

	// Since we have such fancy clipboard monitoring, we don't have
	// to do the url stuff, the clipboard viewer code will do it 
	// for us.
	
	// Add text to clipboard
	if (OpenClipboard()) { 
		::EmptyClipboard();
		// Get length of text
		sLen = listHistory().GetTextLen (nItem);
		// Create buffer for text
		hClip = GlobalAlloc (GHND | GMEM_DDESHARE, sLen+1);
		lpClip = (LPSTR)GlobalLock (hClip);
		// Get the text from the listbox
		listHistory().GetText (nItem, lpClip);

		GlobalUnlock (hClip); 
		if (::SetClipboardData (CF_TEXT, hClip)) {
			// Before we close the clipboard, set flag to
			// allow the url to be translated even if xlate
			// is now disabled.
			m_fEnableOnce = TRUE;
		}
		CloseClipboard(); 
	}
	return;
}
//----------------------------------------------------------------------------
// OnBtnCopy - copies all items into clipboard
//----------------------------------------------------------------------------
void CHistoryPage::OnBtnCopyAll () 
{
	INT i, sLen;
	HGLOBAL hClip;
	LPSTR lpClip;

	if (m_sHistoryCnt == 0)
		return;

	// Get the total length of the items in the history list.
	sLen = 0;
	for (i = 0; i < m_sHistoryCnt; i++) 
		sLen += m_saHistory[i].GetLength();

	if (sLen == 0)
		return;
	if (OpenClipboard()) { 
		::EmptyClipboard();

		hClip = GlobalAlloc (GHND | GMEM_DDESHARE, sLen + 
		                                           (sizeof("\r\n")*m_sHistoryCnt) + 1);
		lpClip = (LPSTR)GlobalLock (hClip);

		for (i = 0; i < m_sHistoryCnt; i++) {
			lstrcpy (lpClip, m_saHistory[i]);
			lstrcat (lpClip, "\r\n");
			lpClip += lstrlen (lpClip);
		}
		GlobalUnlock (hClip); 
		if (::SetClipboardData (CF_TEXT, hClip)) {
			// Before we close the clipboard, set flag to
			// allow the url to be translated even if xlate
			// is now disabled.
			m_fEnableOnce = TRUE;
		} else
			GlobalFree (hClip);
		CloseClipboard(); 
	}
	return;
}
//----------------------------------------------------------------------------
// OnListHistory - Fields double clicks from listbox
//----------------------------------------------------------------------------
void CHistoryPage::OnListHistory () 
{
	PostMessage (WM_COMMAND, MAKELONG (IDD_COPY, BN_CLICKED), 0);
	return;
}
//----------------------------------------------------------------------------
// OnBtnCopy - copies selected listbox item into clipboard
//----------------------------------------------------------------------------
void CHistoryPage::OnBtnCopy () 
{
	INT sSel;

	sSel = listHistory().GetCurSel();
	if (sSel != LB_ERR) 
		CopyItem (sSel);
	return;
}
//----------------------------------------------------------------------------
// OnBtnDelete - Deletes selected listbox item
//----------------------------------------------------------------------------
void CHistoryPage::OnBtnDelete () 
{
	INT sSel;

	// Get current selection and delete it.
	sSel = listHistory().GetCurSel();
	if ((sSel != LB_ERR) && m_sHistoryCnt) {
		m_saHistory.RemoveAt(sSel);
		m_sHistoryCnt--;
		PostMessage (MYMSG_REFRESH, 0, 0);
	}
	return;
}
//----------------------------------------------------------------------------
// OnBtnReset - Clears the history list
//----------------------------------------------------------------------------
void CHistoryPage::OnBtnReset () 
{
	INT rc;

	rc = MessageBox ("Are you sure you want to clear the history list?",
	                 APPNAME, MB_ICONEXCLAMATION | MB_YESNO);
	if (rc == IDYES) {
		m_saHistory.RemoveAll();
		m_sHistoryCnt = 0;
		PostMessage (MYMSG_REFRESH, 0, 0);
	}
	return;
}
//----------------------------------------------------------------------------
// OnRefresh - Updates the history list
//----------------------------------------------------------------------------
LONG CHistoryPage::OnRefresh (UINT wParam, LONG lParam) 
{
	INT i;

	listHistory ().ResetContent();
	// 
	for (i = 0; i < m_sHistoryCnt; i++)
		listHistory ().AddString (m_saHistory[i]);

	if (m_sHistoryCnt) {
		listHistory ().SetCurSel(0);
		btnCopy().EnableWindow (TRUE);
		btnDelete().EnableWindow (TRUE);
		btnReset().EnableWindow (TRUE);
	} else {
		btnCopy().EnableWindow (FALSE);
		btnDelete().EnableWindow (FALSE);
		btnReset().EnableWindow (FALSE);
	}
	return TRUE;
}

//****************************************************************************
//CAboutPage member functions
//
//****************************************************************************

// MFC macro that creates the message dispatch table for the dialog
BEGIN_MESSAGE_MAP (CAboutPage, CPropertyPage)

END_MESSAGE_MAP ()	
				

//----------------------------------------------------------------------------
// OnInitDialog - Called during dialog box init
//----------------------------------------------------------------------------
BOOL CAboutPage::OnInitDialog ()
{
	char szTag[128], szOut[256];
	CWnd *pCtl;
	INT i, rc;

	// create and init version info object
	GetModuleFileName (myApp.m_hInstance, szOut, sizeof (szOut));

	CmyExeVersion VerInfo (szOut);

	for (i = IDD_ABOUT1; i <= IDD_ABOUT7; i++) {
		pCtl = GetDlgItem (i);

		pCtl->GetWindowText (szTag, sizeof (szTag));
 		rc = VerInfo.ParseVerString (szTag, szOut, sizeof (szOut));
		pCtl->SetWindowText (szOut);
	}
	return TRUE;
}
//****************************************************************************
//CTipsDialog member functions
//
//****************************************************************************

//----------------------------------------------------------------------------
// OnInitDialog - Called during dialog box init
//----------------------------------------------------------------------------
BOOL CTipsDialog::OnInitDialog ()
{
	CString	strText;
	CString	strText1;
	CRect rect;

	strText.LoadString (IDS_HELPSTRING1);
	strText1.LoadString (IDS_HELPSTRING2);
	strText += strText1;

	EditTips().GetRect (&rect);
	rect.left += 5;
	rect.right -= 5;
	EditTips().SetRect (&rect);

	EditTips().FmtLines (TRUE);
	EditTips().SetWindowText (strText);

	return TRUE;
}
