//============================================================================
// Include File
//
// Copyright (C) 1997 Ziff-Davis Publishing Company
// First published in PC Magazine by Douglas Boling
//
//============================================================================

//
// program specific typedefs
//
typedef struct {
	INT sCount;
	LPSTR lpEnd;
} HISTSTRUCT;
typedef HISTSTRUCT *LPHISTSTRUCT;

//****************************************************************************
//
// Derive my application class from MFC std app class
//
//****************************************************************************
class CMyApp : public CWinApp
{
public:
	virtual BOOL InitInstance ();
	virtual int ExitInstance ();
	virtual BOOL OnIdle (LONG);
};

//****************************************************************************
//
// Derive Configure dialog sheet
//
//****************************************************************************
#define FLAG_ACTIVE      0x0001
#define FLAG_SAFECLIP    0x0002
#define FLAG_KNOWNONLY   0x0004
class CConfigPage : public CPropertyPage
{
private:
	// macros for control addressability
	CButton& BtnAutoRun () { return *(CButton*) GetDlgItem (IDD_AUTORUN); }
	CButton& BtnSCActivate () { return *(CButton*) GetDlgItem (IDD_SCACTIVATE); }
	
public:
	UINT m_fAutoRun;
	UINT m_fSCActivate;

	INT m_nEnable;
	UINT m_fEnable;
	UINT m_fKnownOnly;
	UINT m_fSafeClip;
	HICON m_hIconEnable;
	HICON m_hIconDisable;
	HWND m_hSheet;

	virtual BOOL OnInitDialog ();
	void Enable(HWND hWnd);

	CConfigPage () : CPropertyPage ("CONFIGPAGE") {}

protected:
	afx_msg void OnBtnClipSafe (UINT);
	afx_msg void OnBtnEnable ();

	afx_msg void OnBtnSCActivate ();
	afx_msg void OnBtnAutoRun ();
	afx_msg void OnBtnRemove ();
	afx_msg void OnBtnHelp ();

	DECLARE_MESSAGE_MAP ()	
};
//****************************************************************************
//
// Derive History dialog sheet 
//
//****************************************************************************
class CHistoryPage : public CPropertyPage
{
private:
	CListBox& listHistory () { return *(CListBox*) GetDlgItem (IDD_HISTORY); }
	CButton& btnCopy () { return *(CButton*) GetDlgItem (IDD_COPY); }
	CButton& btnDelete () { return *(CButton*) GetDlgItem (IDD_DELETE); }
	CButton& btnReset () { return *(CButton*) GetDlgItem (IDD_RESET); }

	afx_msg LONG OnRefresh (UINT, LONG);
public:
	UINT m_fEnableOnce;
	CStringArray m_saHistory;
	INT m_sHistoryCnt;

	virtual BOOL OnInitDialog ();
	void CopyItem (INT);

	CHistoryPage () : CPropertyPage ("HISTORYPAGE") {}

protected:
	afx_msg void OnBtnCopy ();
	afx_msg void OnBtnCopyAll ();
	afx_msg void OnBtnDelete ();
	afx_msg void OnBtnReset ();
	afx_msg void OnListHistory ();

	DECLARE_MESSAGE_MAP ()	
};

//****************************************************************************
//
// Derive my About dialog sheet
//
//****************************************************************************
class CAboutPage : public CPropertyPage
{
private:
//	INT GetClipFormatName (CLIPFORMAT sFmt, PSTR szText, INT sLen);
	INT WhatURL (LPSTR, LPSTR);
	void AddURLToClip (LPSTR lpText, LPSTR lpURL, INT sType);

public:
	virtual BOOL OnInitDialog ();

	CAboutPage () : CPropertyPage ("ABOUTPAGE") {}

protected:

	DECLARE_MESSAGE_MAP ()	
};
typedef struct {
	UINT uFormat;
	HGLOBAL hData;
} MYCLIPENTRY;
typedef MYCLIPENTRY far *LPMYCLIPENTRY;

typedef struct {
	INT sCnt;
	MYCLIPENTRY ce;
} CLIPCACHE;
typedef CLIPCACHE far *LPCLIPCACHE;

//****************************************************************************
//
// Derive my main dialog class
//
//****************************************************************************
class CClipAidSheet : public CPropertySheet
{
private:
	HWND m_hwndNextClip;
	CLIPFORMAT m_cfURL;
	CLIPFORMAT m_cfFileGroupDescriptor;
	CLIPFORMAT m_cfFileContents;
	CLIPFORMAT m_cfFilename;
	CLIPFORMAT m_cfOLEDataObject;
	CLIPFORMAT m_cfRichText;
  	BOOL m_fFlags;

	CConfigPage m_ConfigPage;
	CHistoryPage m_HistoryPage;
	CAboutPage m_AboutPage;

	INT CacheClipData (HGLOBAL *hMem, BOOL fKnownOnly);

	INT GetClipFormatName (CLIPFORMAT sFmt, PSTR szText, INT sLen);
	INT WhatURL (LPSTR, LPSTR);
	void AddURLToClip (LPSTR lpText, LPSTR lpURL, INT sType);

	afx_msg LONG OnTaskBarNotify (UINT, LONG);


public:

	CClipAidSheet (CWnd* = NULL);
	~CClipAidSheet (void);

  	BOOL m_fDirty;
	INT SaveState (void);

	virtual void OnChangeCbChain (HWND, HWND);
	virtual void OnDrawClipboard ();
	afx_msg BOOL DestroyWindow ();

protected:
	virtual BOOL OnInitDialog ();
	virtual void PostNcDestroy ();

	afx_msg void OnClose ();
	virtual void OnCancel ();
	afx_msg void OnMenuShow ();
	afx_msg void OnMenuEnable ();
	afx_msg void OnMenuCopy (UINT);
	afx_msg void OnEndSession (BOOL);

	DECLARE_MESSAGE_MAP ()	
};

//****************************************************************************
//
// Derive Tips dialog class
//
//****************************************************************************
class CTipsDialog : public CDialog
{
private:
	CEdit& EditTips () { return *(CEdit*) GetDlgItem (IDD_TIPTEXT); }

public:
	CTipsDialog(CWnd* pParentWnd = NULL) : CDialog (IDD_TIPSDIALOG, pParentWnd) {}

	virtual BOOL OnInitDialog ();
};

