//
// Version and About Dialog classes
//
// Copyright (C) 1996-1997 Ziff-Davis Publishing Company
// First published in PC Magazine by Douglas Boling
//
// Revision History:
//
// 1.0   Initial Release
//
//****************************************************************************

#include <afx.h>
#include <afxwin.h>

#include "resource.h"  //Grab any resource ids that are needed
#include "VerAbout.h"

//****************************************************************************
//CmyExeVersion member functions
//
//****************************************************************************
//------------------------------------------------------------ 
// Constructor
//------------------------------------------------------------ 
CmyExeVersion::CmyExeVersion (void)
{
	char szFile[MAX_PATH];

	m_hVerInfo = 0;
	GetModuleFileName (AfxGetInstanceHandle(), szFile, sizeof (szFile));
	SetExe (szFile);
}
//
// Init with params.
//
CmyExeVersion::CmyExeVersion (LPCSTR lpExeName, LPSTR lpszKey)
{
	m_hVerInfo = 0;
	SetExe (lpExeName, lpszKey);
}
//------------------------------------------------------------ 
// Destructor
//------------------------------------------------------------ 
CmyExeVersion::~CmyExeVersion (void)
{
	if (m_hVerInfo)
		GlobalFree (m_hVerInfo);
}
//------------------------------------------------------------ 
// Init - Sets params
//------------------------------------------------------------ 
INT CmyExeVersion::SetExe (LPCSTR lpExeName, LPSTR lpszKey)
{
	LPSTR lpVerData;
	DWORD dwHandle;

	if (m_hVerInfo)
		GlobalFree (m_hVerInfo);

	m_strExeName = lpExeName;
	m_strKey = lpszKey;

	// Read version info from file
	m_hVerInfo = 0;
	m_dwVerSize = ::GetFileVersionInfoSize ((LPTSTR)lpExeName, &dwHandle);		
	if (m_dwVerSize) {
		m_hVerInfo = GlobalAlloc (GMEM_MOVEABLE, m_dwVerSize);
		lpVerData = (LPSTR) GlobalLock (m_hVerInfo);
		if (lpVerData) {
			::GetFileVersionInfo ((LPTSTR)lpExeName, dwHandle, m_dwVerSize, lpVerData);

			GlobalUnlock (m_hVerInfo);
		}				
	}		
	return 1;
}
//------------------------------------------------------------ 
// GetVersionTag - Returns the value for a version tag 
//------------------------------------------------------------ 
BOOL CmyExeVersion::GetVersionTag (LPSTR pszTag, LPSTR pszValue, INT *psMax) {
	LPVOID lpVerData;
	INT sKeyLen;
	UINT usLen;
	BOOL rc;
	LPVOID pTemp;

	// If not initialized, return
	if (!m_hVerInfo)
		return 0;

	lpVerData = GlobalLock (m_hVerInfo);
	if (!lpVerData)
		return 0;

	sKeyLen = m_strKey.GetLength();

	// Append tag to end of key string
	m_strKey += pszTag;
	rc = VerQueryValue (lpVerData, m_strKey.GetBuffer(m_strKey.GetLength()+1),
	                    &pTemp, &usLen);

	// Copy data at pointer to output var
	if (rc && (INT)usLen < *psMax) {
		lstrcpy (pszValue, (LPSTR)pTemp);
		*psMax -= (INT)usLen;
	}

	 m_strKey.ReleaseBuffer(-1);
	// Remove tag from end of key.
	m_strKey = m_strKey.Left(sKeyLen);

	GlobalUnlock (m_hVerInfo);
	return rc;
};

//------------------------------------------------------------ 
// GetHardWareName - Returns a string containing the hardware name
//------------------------------------------------------------ 
INT CmyExeVersion::GetHardWareName (LPSTR pszValue, INT *psMax) {

	static char *pszIntel[] = {"386","486", "Pentium","Pentium Pro"};
	char szTemp[80];
	SYSTEM_INFO si;
	INT sLen;

	DWORD dwData = GetVersion ();

	// If not initialized, return
	if (!m_hVerInfo)
		return 0;

	memset (&si, 0, sizeof (si));
	GetSystemInfo (&si);
	switch (si.wProcessorArchitecture) {

		case PROCESSOR_ARCHITECTURE_INTEL:
			// If Win95, don't use full structure
			if ((dwData & 0x80000000) && 
			    (LOBYTE (LOWORD (dwData)) >= 4)) {
				wsprintf (szTemp, "Intel %d", si.dwProcessorType);
			} else {
				lstrcpy (szTemp, "Intel ");
				if ((si.wProcessorLevel > 2) && (si.wProcessorLevel < 7)) 
					lstrcat (szTemp, pszIntel[si.wProcessorLevel-3]);
				else				
					lstrcat (szTemp, "Microprocessor ");

				wsprintf (&szTemp[lstrlen(szTemp)], " %c%d", 
				          (si.wProcessorRevision >> 16) + 'A',
				          si.wProcessorRevision && 0xff);
			}
			break;

		case PROCESSOR_ARCHITECTURE_MIPS:
			wsprintf	(szTemp, "MIPS R%d", si.wProcessorLevel + 1000);
			break;

		case PROCESSOR_ARCHITECTURE_ALPHA:
			wsprintf	(szTemp, "DEC Alpha %d Model %d Pass %d", 
						 si.wProcessorLevel,
						 (si.wProcessorRevision >> 16) + 'A',
						 si.wProcessorRevision && 0xff);
			break;

		case PROCESSOR_ARCHITECTURE_PPC:
			lstrcpy (szTemp, "PowerPC  ");
			switch (si.wProcessorLevel) {
				case 1:
					lstrcat (szTemp, "601");
					break;
				case 3:
					lstrcat (szTemp, "603");
					break;
				case 4:
					lstrcat (szTemp, "604");
					break;
				case 6:
					lstrcat (szTemp, "603+");
					break;
				case 9:
					lstrcat (szTemp, "604+");
					break;
				case 20:
					lstrcat (szTemp, "620");
					break;
			}
			wsprintf	(&szTemp[lstrlen(szTemp)], " %d.%d", 
						 (si.wProcessorRevision >> 16), si.wProcessorRevision && 0xff);
			break;
	}
	*pszValue = 0;
	sLen = lstrlen (szTemp);
	if (sLen < *psMax) {
		lstrcpy (pszValue, szTemp);
		*psMax -= sLen;
	}
	return sLen;
}
//------------------------------------------------------------ 
// GetOSName - Returns a string containing the OS name and ver
//------------------------------------------------------------ 
INT CmyExeVersion::GetOSName (LPSTR pszValue, INT *psMax) {
	char szTemp[80];
	INT sLen;
	DWORD dwData = GetVersion();

	// If not initialized, return
	if (!m_hVerInfo)
		return 0;

	// See if Win NT (high bit 0)
	if (dwData < 0x80000000) {
		wsprintf (szTemp, "Windows NT %u.%u (Build: %u)",
		          (DWORD)(LOBYTE(LOWORD(dwData))),
		          (DWORD)(HIBYTE(LOWORD(dwData))),
		          (DWORD)HIWORD(dwData));

	} else if (LOBYTE (LOWORD (dwData) < 4)) {
		wsprintf (szTemp, "Win32s %u.%u (Build: %u)",
		          (DWORD)(LOBYTE(LOWORD(dwData))),
		          (DWORD)(HIBYTE(LOWORD(dwData))),
		          (DWORD)(HIWORD(dwData) & 0x7fff));
	} else {
		wsprintf (szTemp, "Windows 95 %u.%u",
		          (DWORD)(LOBYTE(LOWORD(dwData))),
		          (DWORD)(HIBYTE(LOWORD(dwData))));
	}
	*pszValue = 0;
	sLen = lstrlen (szTemp);
	if ((INT)sLen < *psMax) {
		lstrcpy (pszValue, szTemp);
		*psMax -= sLen;
	}
	return sLen;
}
//------------------------------------------------------------ 
// ParseAboutStr - Parses about box string to include version
// information.
// Note: Strings enclosed in percent signs are replaced with
//       version info from the rc file.  
//       ~O is replaced with the current operating system
//       ~T is replaced with the target operating system
//       ~H is replaced with the target hardware
//------------------------------------------------------------ 
INT CmyExeVersion::ParseVerString (LPSTR lpszIn, LPSTR lpszOut, INT sMax) {
	LPSTR lpVerData, pszTag;
	LPSTR lpszStart;

	INT i;

	if (!lpszOut)
		return 0;

	*lpszOut = '\0';
	// If not initialized, return
	if (!m_hVerInfo) {
		if (lstrlen (lpszIn) < sMax)
			lstrcpy (lpszOut, lpszIn);
		return 0;
	}
	lpVerData = (LPSTR)GlobalLock (m_hVerInfo);
	if (!lpVerData)
		return 0;

	lpszStart = lpszOut;

	while (*lpszIn && sMax) {
		// See if start of version info key
		if (*lpszIn == '%') {
			lpszIn++;      //Skip past initial %
			if (*lpszIn != '%') {
			
				pszTag = lpszIn;

				// Find end of tag
				for (i = 0; *lpszIn && (*lpszIn != '%'); i++) 
					lpszIn++;
				*lpszIn = '\0';
				// Get version info. if this fails, append tag name
				if (!GetVersionTag (pszTag, lpszOut, &sMax)) {
					if (lstrlen (pszTag)+2 < sMax) {
						lstrcat (lpszOut, pszTag-1);
						lstrcat (lpszOut, "%");
					}						
				}
				lpszOut += lstrlen (lpszOut);
			} else {
				*lpszOut++ = '%';
				sMax--;
			}
			lpszIn++;      //Skip past trailing %

		// if not ver tag, see if switch for OS info
		} else if (*lpszIn == '~') {
			lpszIn++;      //Skip past ~
			switch (*lpszIn) {

				// Hardware platform
				case 'H':
				case 'h':
					GetHardWareName (lpszOut, &sMax);
					lpszOut += lstrlen (lpszOut);
					break;

				// Target OS
				case 'T':
				case 't':
					if (sMax > 11) {
#if (WINVER < 0x400)
						lstrcpy (lpszOut, "Windows NT");
#else					
						lstrcpy (lpszOut, "Windows 95");
#endif
						lpszOut += lstrlen (lpszOut);

					}
					break;

				// Current OS
				case 'O':
				case 'o':
					GetOSName (lpszOut, &sMax);
					lpszOut += lstrlen (lpszOut);
					break;
				 
				// If unrecognised, insert bad tag in output to alert user
				default:
					if (sMax > 2) {
						*lpszOut++ = '~';
						*lpszOut++ = *lpszIn;
					}
					break;	
			}					
			lpszIn++;
		} else {
			*lpszOut++ = *lpszIn++;
			sMax--;
		}
	}	
	GlobalUnlock (m_hVerInfo);
	*lpszOut = '\0';
	return lstrlen (lpszStart);
};
